unit uGPIOBibliothek;

(* Routinen für den Zugriff auf die GPIO PINs, Initialisieren, Schalten
   der Relaiskarten und der SSRs etc. *)

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Crt, Unix, BaseUnix, Dialogs, uGlobalVar, uConfig, uLog;


procedure GPIOInit (PIN : pChar);
procedure GPIOSetForOutput (PIN : pChar);
procedure GPIOSetForInput (PIN : pChar);
procedure GPIOUnexport (PIN : pChar);
procedure GPIOSwitchRelais (PIN : pChar; PINOn : boolean);
procedure GPIOGetSchalterStatus (PIN : pChar; var SchalterStatus : boolean);


implementation


procedure GPIOSwitchRelais (PIN : pChar; PINOn : boolean);
var
  fileDesc: integer;
  sPIN : string2;

begin
  sPIN := PIN;

  if PINOn then
  begin
    { Relais An (Low aktiv) }

    try
      fileDesc := fpopen('/sys/class/gpio/gpio'+sPIN+'/value', O_WrOnly);
      gReturnCode := fpwrite(fileDesc, PIN_OFF[0], 1);
      LogWrite ('Relais ON '+sPIN,gReturnCode);
    finally
      gReturnCode := fpclose(fileDesc);
      LogWrite ('Close',gReturnCode);
    end;
  end
  else
  begin
    { Relais aus }
    try
      fileDesc := fpopen('/sys/class/gpio/gpio'+sPIN+'/value', O_WrOnly);
      gReturnCode := fpwrite(fileDesc, PIN_ON[0], 1);
      LogWrite ('Relais OFF '+sPIN,gReturnCode);
    finally
      gReturnCode := fpclose(fileDesc);
      LogWrite ('Close',gReturnCode);
    end;
  end;

  (* Hier ist das Problem mit dem SIGSEGV Fehler
     Geht zwar auch mit Konstante, aber packe lieber alles auf die Form config,
     da beim ersten Initialsieren viele Pumpen , SSRs, Schütze etc. kurz hintereiander
     schalten. Daher besser längeres delay sinnvoll, später im Betrieb kleines delay
   *)

  IF config <> nil
     then delay (Config.SpinEditSwitchDelay.Value)
     else delay (80);
end;



procedure GPIOGetSchalterStatus (PIN : pChar; var SchalterStatus : boolean);

var
  fileDesc: integer;
  sPIN : string2;
  ButtonStatus : string [1] = '1';

begin
  sPIN := PIN;

  try
    fileDesc := fpopen('/sys/class/gpio/gpio'+sPIN+'/value', O_RdOnly);

    if fileDesc > 0 then
       begin
         gReturnCode := fpread (fileDesc, ButtonStatus [1], 1);
         LogWrite ('Read Switch'+SPIN,gReturnCode);

         if ButtonStatus = '0'
            then SchalterStatus := true
            else SchalterStatus := false;
       end;
  finally
      gReturnCode := fpclose(fileDesc);
      LogWrite ('Close',gReturnCode);
  end;
end;





procedure GPIOInit (PIN : pChar);
var
  fileDesc: integer;

begin

  { Prepare SoC pin 17 (pin 11 on GPIO port) for access: }
  try
    fileDesc := fpopen('/sys/class/gpio/export', O_WrOnly);
    gReturnCode := fpwrite(fileDesc, PIN[0], 2);
    LogWrite ('GPIO Init',gReturnCode);
  finally
    gReturnCode := fpclose(fileDesc);
    LogWrite ('Close',gReturnCode);
  end;

  // Hier manchmal der SIGSEGV Fehler

  IF config <> nil
     then delay (Config.SpinEditInitDelay.Value)
     else delay (80);
end;


procedure GPIOSetForOutput (PIN : pChar);
  var
    fileDesc: integer;
    sPIN : string2;

begin
  sPIN := PIN;

  { Set SoC pin 17 as output: }
  try
    fileDesc := fpopen('/sys/class/gpio/gpio'+sPIN+'/direction', O_WrOnly);
    gReturnCode := fpwrite(fileDesc, OUT_DIRECTION[0], 3);
    LogWrite ('GPIO SetFOROutput'+SPIN,gReturnCode);
  finally
    gReturnCode := fpclose(fileDesc);
    LogWrite ('Close '+SPIN,gReturnCode);
  end;

  // hier selten das gleiche Problem

  IF config <> nil
     then delay (Config.SpinEditInitDelay.Value)
     else delay (80);
end;


procedure GPIOSetForInput (PIN : pChar);
type string2 = string [2];
  var
    fileDesc: integer;
    sPIN : string2;
begin
   sPIN := PIN;

  { Set SoC pin 17 as output: }
  try
    fileDesc := fpopen('/sys/class/gpio/gpio'+sPIN+'/direction', O_WrOnly);
    gReturnCode := fpwrite(fileDesc, IN_DIRECTION[0], 2);
    LogWrite ('GPIOSetForInput '+SPIN,gReturnCode);
  finally
    gReturnCode := fpclose(fileDesc);
    LogWrite ('Close '+SPIN,gReturnCode);
  end;
end;


procedure GPIOUnexport (PIN : pChar);
var
  fileDesc: integer;

begin
  try
    fileDesc := fpopen('/sys/class/gpio/unexport', O_WrOnly);
    gReturnCode := fpwrite(fileDesc, PIN[0], 2);
    LogWrite ('GPIOUnExport',gReturnCode);
  finally
    gReturnCode := fpclose(fileDesc);
    LogWrite ('Close',gReturnCode);
  end;
end;



end.

